@@ -1,7 +1,7 @@
 # coding: utf-8
 # /*##########################################################################
 #
-# Copyright (c) 2004-2016 European Synchrotron Radiation Facility
+# Copyright (c) 2004-2017 European Synchrotron Radiation Facility
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -29,7 +29,7 @@ This widget is meant to work with :class:`PlotWindow`.
 
 __authors__ = ["V.A. Sole", "T. Rueter", "T. Vincent"]
 __license__ = "MIT"
-__data__ = "28/04/2016"
+__data__ = "08/08/2016"
 
 
 import logging
@@ -259,6 +259,7 @@ class LegendModel(qt.QAbstractListModel):
             legendList = []
         self.legendList = []
         self.insertLegendList(0, legendList)
+        self._palette = qt.QPalette()
 
     def __getitem__(self, idx):
         if idx >= len(self.legendList):
@@ -282,6 +283,7 @@ class LegendModel(qt.QAbstractListModel):
             raise IndexError('list index out of range')
 
         item = self.legendList[idx]
+        isActive = item[1].get("active", False)
         if role == qt.Qt.DisplayRole:
             # Data to be rendered in the form of text
             legend = str(item[0])
@@ -295,14 +297,19 @@ class LegendModel(qt.QAbstractListModel):
             return alignment
         elif role == qt.Qt.BackgroundRole:
             # Background color, must be QBrush
-            if idx % 2:
+            if isActive:
+                brush = self._palette.brush(qt.QPalette.Normal, qt.QPalette.Highlight)
+            elif idx % 2:
                 brush = qt.QBrush(qt.QColor(240, 240, 240))
             else:
                 brush = qt.QBrush(qt.Qt.white)
             return brush
         elif role == qt.Qt.ForegroundRole:
             # ForegroundRole color, must be QBrush
-            brush = qt.QBrush(qt.Qt.blue)
+            if isActive:
+                brush = self._palette.brush(qt.QPalette.Normal, qt.QPalette.HighlightedText)
+            else:
+                brush = self._palette.brush(qt.QPalette.Normal, qt.QPalette.WindowText)
             return brush
         elif role == qt.Qt.CheckStateRole:
             return bool(item[2])  # item[2] == True
@@ -513,6 +520,7 @@ class LegendListItemWidget(qt.QItemDelegate):
         textAlign = modelIndex.data(qt.Qt.TextAlignmentRole)
         painter.setBrush(textBrush)
         painter.setFont(self.legend.font())
+        painter.setPen(textBrush.color())
         painter.drawText(labelRect, textAlign, legendText)
 
         # Draw icon
@@ -614,7 +622,7 @@ class LegendListView(qt.QListView):
         self.setSelectionMode(qt.QAbstractItemView.NoSelection)
 
         if model is None:
-            model = LegendModel()
+            model = LegendModel(parent=self)
         self.setModel(model)
         self.setContextMenu(contextMenu)
 
@@ -966,9 +974,24 @@ class LegendsDockWidget(qt.QDockWidget):
         :param str oldLegend: The legend of the curve to be change
         :param str newLegend: The new legend of the curve
         """
-        x, y, _legend, _info, params = self.plot.getCurve(oldLegend)[0:5]
+        curve = self.plot.getCurve(oldLegend)
         self.plot.remove(oldLegend, kind='curve')
-        self.plot.addCurve(x, y, legend=newLegend, resetzoom=False, **params)
+        self.plot.addCurve(curve.getXData(copy=False),
+                           curve.getYData(copy=False),
+                           legend=newLegend,
+                           info=curve.getInfo(),
+                           color=curve.getColor(),
+                           symbol=curve.getSymbol(),
+                           linewidth=curve.getLineWidth(),
+                           linestyle=curve.getLineStyle(),
+                           xlabel=curve.getXLabel(),
+                           ylabel=curve.getYLabel(),
+                           xerror=curve.getXErrorData(copy=False),
+                           yerror=curve.getYErrorData(copy=False),
+                           z=curve.getZValue(),
+                           selectable=curve.isSelectable(),
+                           fill=curve.isFill(),
+                           resetzoom=False)
 
     def _legendSignalHandler(self, ddict):
         """Handles events from the LegendListView signal"""
@@ -998,27 +1021,33 @@ class LegendsDockWidget(qt.QDockWidget):
 
         elif ddict['event'] in ["mapToRight", "mapToLeft"]:
             legend = ddict['legend']
-            x, y, legend, _info, params = self.plot.getCurve(legend)[0:5]
-            params = params.copy()
-            if ddict['event'] == "mapToRight":
-                params['yaxis'] = "right"
-            else:
-                params['yaxis'] = "left"
-            self.plot.addCurve(x, y, legend=legend, **params)
+            curve = self.plot.getCurve(legend)
+            yaxis = 'right' if ddict['event'] == 'mapToRight' else 'left'
+            self.plot.addCurve(x=curve.getXData(copy=False),
+                               y=curve.getYData(copy=False),
+                               legend=curve.getLegend(),
+                               info=curve.getInfo(),
+                               yaxis=yaxis)
 
         elif ddict['event'] == "togglePoints":
             legend = ddict['legend']
-            x, y, legend, _info, params = self.plot.getCurve(legend)[0:5]
-            params = params.copy()
-            params['symbol'] = ddict['symbol'] if ddict['points'] else ''
-            self.plot.addCurve(x, y, legend=legend, resetzoom=False, **params)
+            curve = self.plot.getCurve(legend)
+            symbol = ddict['symbol'] if ddict['points'] else ''
+            self.plot.addCurve(x=curve.getXData(copy=False),
+                               y=curve.getYData(copy=False),
+                               legend=curve.getLegend(),
+                               info=curve.getInfo(),
+                               symbol=symbol)
 
         elif ddict['event'] == "toggleLine":
             legend = ddict['legend']
-            x, y, legend, _info, params = self.plot.getCurve(legend)[0:5]
-            params = params.copy()
-            params['linestyle'] = ddict['linestyle'] if ddict['line'] else ''
-            self.plot.addCurve(x, y, legend=legend, resetzoom=False, **params)
+            curve = self.plot.getCurve(legend)
+            linestyle = ddict['linestyle'] if ddict['line'] else ''
+            self.plot.addCurve(x=curve.getXData(copy=False),
+                               y=curve.getYData(copy=False),
+                               legend=curve.getLegend(),
+                               info=curve.getInfo(),
+                               linestyle=linestyle)
 
         else:
             _logger.debug("unhandled event %s", str(ddict['event']))
@@ -1027,20 +1056,23 @@ class LegendsDockWidget(qt.QDockWidget):
         """Sync the LegendSelector widget displayed info with the plot.
         """
         legendList = []
-        curves = self.plot.getAllCurves(withhidden=True)
-        for x, y, legend, _info, params in curves:
+        for curve in self.plot.getAllCurves(withhidden=True):
+            legend = curve.getLegend()
             # Use active color if curve is active
             if legend == self.plot.getActiveCurve(just_legend=True):
-                color = self.plot.getActiveCurveColor()
+                color = qt.QColor(self.plot.getActiveCurveColor())
+                isActive = True
             else:
-                color = params['color']
+                color = qt.QColor.fromRgbF(*curve.getColor())
+                isActive = False
 
             curveInfo = {
-                'color': qt.QColor(color),
-                'linewidth': params['linewidth'],
-                'linestyle': params['linestyle'],
-                'symbol': params['symbol'],
-                'selected': not self.plot.isCurveHidden(legend)}
+                'color': color,
+                'linewidth': curve.getLineWidth(),
+                'linestyle': curve.getLineStyle(),
+                'symbol': curve.getSymbol(),
+                'selected': not self.plot.isCurveHidden(legend),
+                'active': isActive}
             legendList.append((legend, curveInfo))
 
         self._legendWidget.setLegendList(legendList)
@@ -1057,3 +1089,10 @@ class LegendsDockWidget(qt.QDockWidget):
                 self.plot.sigContentChanged.disconnect(self.updateLegends)
                 self.plot.sigActiveCurveChanged.disconnect(self.updateLegends)
                 self._isConnected = False
+
+    def showEvent(self, event):
+        """Make sure this widget is raised when it is shown
+        (when it is first created as a tab in PlotWindow or when it is shown
+        again after hiding).
+        """
+        self.raise_()
