@@ -28,18 +28,22 @@ and stacks of images"""
 
 __authors__ = ["V.A. Sole", "T. Vincent", "P. Knobel", "H. Payno"]
 __license__ = "MIT"
-__date__ = "11/01/2017"
+__date__ = "17/08/2017"
 
 
-import numpy 
+import numpy
 
 from silx.image.bilinear import BilinearImage
 
 from .. import icons
 from .. import qt
+from . import items
 from .Colors import cursorColorForColormap
-from .PlotActions import PlotAction
+from . import actions
 from .PlotToolButtons import ProfileToolButton
+from .ProfileMainWindow import ProfileMainWindow
+
+from silx.utils.deprecation import deprecated
 
 
 def _alignedFullProfile(data, origin, scale, position, roiWidth, axis):
@@ -133,7 +137,7 @@ def _alignedPartialProfile(data, rowRange, colRange, axis):
     colEnd = min(max(0, colRange[1]), width)
 
     imgProfile = numpy.mean(data[:, rowStart:rowEnd, colStart:colEnd],
-                            axis=axis+1, dtype=numpy.float32)
+                            axis=axis + 1, dtype=numpy.float32)
 
     # Profile including out of bound area
     profile = numpy.zeros((nimages, profileLength), dtype=numpy.float32)
@@ -145,15 +149,17 @@ def _alignedPartialProfile(data, rowRange, colRange, axis):
     return profile
 
 
-def createProfile(roiInfo, currentData, params, lineWidth):
+def createProfile(roiInfo, currentData, origin, scale, lineWidth):
     """Create the profile line for the the given image.
 
     :param roiInfo: information about the ROI: start point, end point and
         type ("X", "Y", "D")
     :param numpy.ndarray currentData: the 2D image or the 3D stack of images
         on which we compute the profile.
-    :param dict params: parameters of the plot, such as origin, scale
-        and colormap
+    :param origin: (ox, oy) the offset from origin
+    :type origin: 2-tuple of float
+    :param scale: (sx, sy) the scale to use
+    :type scale: 2-tuple of float
     :param int lineWidth: width of the profile line
     :return: `profile, area, profileName, xLabel`, where:
         - profile is a 2D array of the profiles of the stack of images.
@@ -168,9 +174,8 @@ def createProfile(roiInfo, currentData, params, lineWidth):
 
     :rtype: tuple(ndarray, (ndarray, ndarray), str, str)
     """
-    if currentData is None or params is None or\
-        roiInfo is None or lineWidth is None:
-        return
+    if currentData is None or roiInfo is None or lineWidth is None:
+        raise ValueError("createProfile called with invalide arguments")
 
     # force 3D data (stack of images)
     if len(currentData.shape) == 2:
@@ -178,8 +183,6 @@ def createProfile(roiInfo, currentData, params, lineWidth):
     elif len(currentData.shape) == 3:
         currentData3D = currentData
 
-    origin, scale = params['origin'], params['scale']
-
     roiWidth = max(1, lineWidth)
     roiStart, roiEnd, lineProjectionMode = roiInfo
 
@@ -266,12 +269,11 @@ def createProfile(roiInfo, currentData, params, lineWidth):
                 bilinear = BilinearImage(currentData3D[slice_idx, :, :])
 
                 profile.append(bilinear.profile_line(
-                                        (startPt[0] - 0.5, startPt[1] - 0.5),
-                                        (endPt[0] - 0.5, endPt[1] - 0.5),
-                                        roiWidth))
+                    (startPt[0] - 0.5, startPt[1] - 0.5),
+                    (endPt[0] - 0.5, endPt[1] - 0.5),
+                    roiWidth))
             profile = numpy.array(profile)
 
-
             # Extend ROI with half a pixel on each end, and
             # Convert back to plot coords (x, y)
             length = numpy.sqrt((endPt[0] - startPt[0]) ** 2 +
@@ -318,8 +320,7 @@ class ProfileToolBar(qt.QToolBar):
 
     Attributes:
 
-    - plot: Associated :class:`PlotWindow`.
-    - profileWindow: Associated :class:`PlotWindow` displaying the profile.
+    - plot: Associated :class:`PlotWindow` on which the profile line is drawn.
     - actionGroup: :class:`QActionGroup` of available actions.
 
     To run the following sample code, a QApplication must be initialized.
@@ -327,7 +328,6 @@ class ProfileToolBar(qt.QToolBar):
 
     >>> from silx.gui.plot import PlotWindow
     >>> from silx.gui.plot.Profile import ProfileToolBar
-    >>> from silx.gui import qt
 
     >>> plot = PlotWindow()  # Create a PlotWindow
     >>> toolBar = ProfileToolBar(plot=plot)  # Create a profile toolbar
@@ -335,7 +335,7 @@ class ProfileToolBar(qt.QToolBar):
     >>> plot.show()  # To display the PlotWindow with the profile toolbar
 
     :param plot: :class:`PlotWindow` instance on which to operate.
-    :param profileWindow: :class:`ProfileScanWidget` instance where to
+    :param profileWindow: Plot widget instance where to
                           display the profile curve or None to create one.
     :param str title: See :class:`QToolBar`.
     :param parent: See :class:`QToolBar`.
@@ -355,67 +355,67 @@ class ProfileToolBar(qt.QToolBar):
 
         self._roiInfo = None  # Store start and end points and type of ROI
 
-        if profileWindow is None:
-            # Import here to avoid cyclic import
-            from .PlotWindow import Plot1D  # noqa
-            self.profileWindow = Plot1D()
-            self._ownProfileWindow = True
-        else:
-            self.profileWindow = profileWindow
-            self._ownProfileWindow = False
+        self._profileWindow = profileWindow
+        """User provided plot widget in which the profile curve is plotted.
+        None if no custom profile plot was provided."""
+
+        self._profileMainWindow = None
+        """Main window providing 2 profile plot widgets for 1D or 2D profiles.
+        The window provides two public methods
+            - :meth:`setProfileDimensions`
+            - :meth:`getPlot`: return handle on the actual plot widget
+              currently being used
+        None if the user specified a custom profile plot window.
+        """
+
+        if self._profileWindow is None:
+            self._profileMainWindow = ProfileMainWindow(self)
 
         # Actions
-        self.browseAction = qt.QAction(
-                icons.getQIcon('normal'),
-                'Browsing Mode', None)
-        self.browseAction.setToolTip(
-                'Enables zooming interaction mode')
-        self.browseAction.setCheckable(True)
-        self.browseAction.triggered[bool].connect(self._browseActionTriggered)
+        self._browseAction = actions.mode.ZoomModeAction(self.plot, parent=self)
+        self._browseAction.setVisible(False)
 
         self.hLineAction = qt.QAction(
-                icons.getQIcon('shape-horizontal'),
-                'Horizontal Profile Mode', None)
+            icons.getQIcon('shape-horizontal'),
+            'Horizontal Profile Mode', None)
         self.hLineAction.setToolTip(
-                'Enables horizontal profile selection mode')
+            'Enables horizontal profile selection mode')
         self.hLineAction.setCheckable(True)
         self.hLineAction.toggled[bool].connect(self._hLineActionToggled)
 
         self.vLineAction = qt.QAction(
-                icons.getQIcon('shape-vertical'),
-                'Vertical Profile Mode', None)
+            icons.getQIcon('shape-vertical'),
+            'Vertical Profile Mode', None)
         self.vLineAction.setToolTip(
-                'Enables vertical profile selection mode')
+            'Enables vertical profile selection mode')
         self.vLineAction.setCheckable(True)
         self.vLineAction.toggled[bool].connect(self._vLineActionToggled)
 
         self.lineAction = qt.QAction(
-                icons.getQIcon('shape-diagonal'),
-                'Free Line Profile Mode', None)
+            icons.getQIcon('shape-diagonal'),
+            'Free Line Profile Mode', None)
         self.lineAction.setToolTip(
-                'Enables line profile selection mode')
+            'Enables line profile selection mode')
         self.lineAction.setCheckable(True)
         self.lineAction.toggled[bool].connect(self._lineActionToggled)
 
         self.clearAction = qt.QAction(
-                icons.getQIcon('image'),
-                'Clear Profile', None)
+            icons.getQIcon('profile-clear'),
+            'Clear Profile', None)
         self.clearAction.setToolTip(
-                'Clear the profile Region of interest')
+            'Clear the profile Region of interest')
         self.clearAction.setCheckable(False)
         self.clearAction.triggered.connect(self.clearProfile)
 
         # ActionGroup
         self.actionGroup = qt.QActionGroup(self)
-        self.actionGroup.addAction(self.browseAction)
+        self.actionGroup.addAction(self._browseAction)
         self.actionGroup.addAction(self.hLineAction)
         self.actionGroup.addAction(self.vLineAction)
         self.actionGroup.addAction(self.lineAction)
 
-        self.browseAction.setChecked(True)
-
         # Add actions to ToolBar
-        self.addAction(self.browseAction)
+        self.addAction(self._browseAction)
         self.addAction(self.hLineAction)
         self.addAction(self.vLineAction)
         self.addAction(self.lineAction)
@@ -427,40 +427,64 @@ class ProfileToolBar(qt.QToolBar):
         self.lineWidthSpinBox.setRange(0, 1000)
         self.lineWidthSpinBox.setValue(1)
         self.lineWidthSpinBox.valueChanged[int].connect(
-                self._lineWidthSpinBoxValueChangedSlot)
+            self._lineWidthSpinBoxValueChangedSlot)
         self.addWidget(self.lineWidthSpinBox)
 
         self.plot.sigInteractiveModeChanged.connect(
-                self._interactiveModeChanged)
+            self._interactiveModeChanged)
 
         # Enable toolbar only if there is an active image
         self.setEnabled(self.plot.getActiveImage(just_legend=True) is not None)
         self.plot.sigActiveImageChanged.connect(
-                self._activeImageChanged)
+            self._activeImageChanged)
 
-        # will manage the close event
-        self.profileWindow.installEventFilter(self)
+        # listen to the profile window signals to clear profile polygon on close
+        if self.getProfileMainWindow() is not None:
+            self.getProfileMainWindow().sigClose.connect(self.clearProfile)
 
-    def eventFilter(self, qobject, event):
-        """Observe when the close event is emitted to clear the profile
+    @property
+    @deprecated(since_version="0.6.0")
+    def browseAction(self):
+        return self._browseAction
 
-        :param qobject: the object observe
-        :param event: the event received by qobject
+    @property
+    @deprecated(replacement="getProfilePlot", since_version="0.5.0")
+    def profileWindow(self):
+        return self.getProfilePlot()
+
+    def getProfilePlot(self):
+        """Return plot widget in which the profile curve or the
+        profile image is plotted.
         """
-        if event.type() in (qt.QEvent.Close, qt.QEvent.Hide):
-            self.clearProfile()
+        if self.getProfileMainWindow() is not None:
+            return self.getProfileMainWindow().getPlot()
 
-        return qt.QToolBar.eventFilter(self, qobject, event)
+        # in case the user provided a custom plot for profiles
+        return self._profileWindow
+
+    def getProfileMainWindow(self):
+        """Return window containing the profile curve widget.
+        This can return *None* if a custom profile plot window was
+        specified in the constructor.
+        """
+        return self._profileMainWindow
 
     def _activeImageChanged(self, previous, legend):
         """Handle active image change: toggle enabled toolbar, update curve"""
-        self.setEnabled(legend is not None)
-        if legend is not None:
-            # Update default profile color
+        if legend is None:
+            self.setEnabled(False)
+        else:
             activeImage = self.plot.getActiveImage()
-            if activeImage is not None:
+
+            # Disable for empty image
+            self.setEnabled(activeImage.getData(copy=False).size > 0)
+
+            # Update default profile color
+            if isinstance(activeImage, items.ColormapMixIn):
                 self._defaultOverlayColor = cursorColorForColormap(
-                        activeImage[4]['colormap']['name'])
+                    activeImage.getColormap()['name'])
+            else:
+                self._defaultOverlayColor = 'black'
 
             self.updateProfile()
 
@@ -474,7 +498,15 @@ class ProfileToolBar(qt.QToolBar):
         If changed from elsewhere, disable drawing tool
         """
         if source is not self:
-            self.browseAction.setChecked(True)
+            self.clearProfile()
+
+            # Uncheck all drawing profile modes
+            self.hLineAction.setChecked(False)
+            self.vLineAction.setChecked(False)
+            self.lineAction.setChecked(False)
+
+            if self.getProfileMainWindow() is not None:
+                self.getProfileMainWindow().hide()
 
     def _hLineActionToggled(self, checked):
         """Handle horizontal line profile action toggle"""
@@ -503,13 +535,6 @@ class ProfileToolBar(qt.QToolBar):
         else:
             self.plot.sigPlotSignal.disconnect(self._plotWindowSlot)
 
-    def _browseActionTriggered(self, checked):
-        """Handle browse action mode triggered by user."""
-        if checked:
-            self.clearProfile()
-            self.plot.setInteractiveMode('zoom', source=self)
-            self.profileWindow.hide()
-
     def _plotWindowSlot(self, event):
         """Listen to Plot to handle drawing events to refresh ROI and profile.
         """
@@ -555,66 +580,76 @@ class ProfileToolBar(qt.QToolBar):
 
         This uses the current active image of the plot and the current ROI.
         """
-        imageData = self.plot.getActiveImage()
-        if imageData is None:
+        image = self.plot.getActiveImage()
+        if image is None:
             return
 
         # Clean previous profile area, and previous curve
         self.plot.remove(self._POLYGON_LEGEND, kind='item')
-        self.profileWindow.clear()
-        self.profileWindow.setGraphTitle('')
-        self.profileWindow.setGraphXLabel('X')
-        self.profileWindow.setGraphYLabel('Y')
+        self.getProfilePlot().clear()
+        self.getProfilePlot().setGraphTitle('')
+        self.getProfilePlot().getXAxis().setLabel('X')
+        self.getProfilePlot().getYAxis().setLabel('Y')
 
-        self._createProfile(currentData=imageData[0], params=imageData[4])
+        self._createProfile(currentData=image.getData(copy=False),
+                            origin=image.getOrigin(),
+                            scale=image.getScale(),
+                            colormap=None,  # Not used for 2D data
+                            z=image.getZValue())
 
-    def _createProfile(self, currentData, params):
+    def _createProfile(self, currentData, origin, scale, colormap, z):
         """Create the profile line for the the given image.
 
         :param numpy.ndarray currentData: the image or the stack of images
             on which we compute the profile
-        :param params: parameters of the plot, such as origin, scale
-            and colormap
+        :param origin: (ox, oy) the offset from origin
+        :type origin: 2-tuple of float
+        :param scale: (sx, sy) the scale to use
+        :type scale: 2-tuple of float
+        :param dict colormap: The colormap to use
+        :param int z: The z layer of the image
         """
-        assert ('colormap' in params and 'z' in params)
         if self._roiInfo is None:
             return
 
         profile, area, profileName, xLabel = createProfile(
-                roiInfo=self._roiInfo,
-                currentData=currentData,
-                params=params,
-                lineWidth=self.lineWidthSpinBox.value())
-        colorMap = params['colormap']
+            roiInfo=self._roiInfo,
+            currentData=currentData,
+            origin=origin,
+            scale=scale,
+            lineWidth=self.lineWidthSpinBox.value())
 
-        self.profileWindow.setGraphTitle(profileName)
+        self.getProfilePlot().setGraphTitle(profileName)
 
         dataIs3D = len(currentData.shape) > 2
         if dataIs3D:
-            self.profileWindow.addImage(profile,
-                                        legend=profileName,
-                                        xlabel=xLabel,
-                                        ylabel="Frame index (depth)",
-                                        colormap=colorMap)
+            self.getProfilePlot().addImage(profile,
+                                           legend=profileName,
+                                           xlabel=xLabel,
+                                           ylabel="Frame index (depth)",
+                                           colormap=colormap)
         else:
             coords = numpy.arange(len(profile[0]), dtype=numpy.float32)
-            self.profileWindow.addCurve(coords, profile[0],
-                                        legend=profileName,
-                                        xlabel=xLabel,
-                                        color=self.overlayColor)
+            self.getProfilePlot().addCurve(coords,
+                                           profile[0],
+                                           legend=profileName,
+                                           xlabel=xLabel,
+                                           color=self.overlayColor)
 
         self.plot.addItem(area[0], area[1],
                           legend=self._POLYGON_LEGEND,
                           color=self.overlayColor,
                           shape='polygon', fill=True,
-                          replace=False, z=params['z'] + 1)
+                          replace=False, z=z + 1)
 
-        self._showProfileWindow()
+        self._showProfileMainWindow()
 
-    def _showProfileWindow(self):
-        """If profile window was created in this widget,
-        it tries to avoid overlapping this widget when shown"""
-        if self._ownProfileWindow and not self.profileWindow.isVisible():
+    def _showProfileMainWindow(self):
+        """If profile window was created by this toolbar,
+        try to avoid overlapping with the toolbar's parent window.
+        """
+        profileMainWindow = self.getProfileMainWindow()
+        if profileMainWindow is not None:
             winGeom = self.window().frameGeometry()
             qapp = qt.QApplication.instance()
             screenGeom = qapp.desktop().availableGeometry(self)
@@ -622,228 +657,92 @@ class ProfileToolBar(qt.QToolBar):
             spaceOnLeftSide = winGeom.left()
             spaceOnRightSide = screenGeom.width() - winGeom.right()
 
-            profileWindowWidth = self.profileWindow.frameGeometry().width()
+            profileWindowWidth = profileMainWindow.frameGeometry().width()
             if (profileWindowWidth < spaceOnRightSide or
-                        spaceOnRightSide > spaceOnLeftSide):
+                    spaceOnRightSide > spaceOnLeftSide):
                 # Place profile on the right
-                self.profileWindow.move(winGeom.right(), winGeom.top())
+                profileMainWindow.move(winGeom.right(), winGeom.top())
             else:
                 # Not enough place on the right, place profile on the left
-                self.profileWindow.move(
+                profileMainWindow.move(
                         max(0, winGeom.left() - profileWindowWidth), winGeom.top())
 
-        self.profileWindow.show()
+            profileMainWindow.show()
+        else:
+            self.getProfilePlot().show()
 
     def hideProfileWindow(self):
         """Hide profile window.
         """
-        self.profileWindow.hide()
-
-
-class Profile3DAction(PlotAction):
-    """PlotAction that emits a signal when checked, to notify
-
-    :param plot: :class:`.PlotWidget` instance on which to operate.
-    :param parent: See :class:`QAction`.
-    """
-    sigProfileDimensionChanged = qt.Signal(int)
-
-    def __init__(self, plot, parent=None):
-        # Uses two images for checked/unchecked states
-        self._states = {
-            1: (icons.getQIcon('profile1D'),
-                    "Compute 1D profile"),
-            2: (icons.getQIcon('profile2D'),
-                   "Compute 2D profile")
-        }
-
-        icon, tooltip = self._states[True]
-        super(Profile3DAction, self).__init__(
-                plot=plot,
-                icon=icon,
-                text='Profile',
-                tooltip=tooltip,
-                triggered=self.__compute3DProfile,
-                checkable=False,
-                parent=parent)
-
-    def __compute3DProfile(self, profileDimension):
-        """Callback when the QAction is activated
-        """
-        icon, tooltip = self._states[profileDimension]
-        self.setIcon(icon)
-        self.setToolTip(tooltip)
-        self.sigProfileDimensionChanged.emit(profileDimension)
+        # this method is currently only used by StackView when the perspective
+        # is changed
+        if self.getProfileMainWindow() is not None:
+            self.getProfileMainWindow().hide()
 
 
 class Profile3DToolBar(ProfileToolBar):
-    def __init__(self, parent=None, plot=None, title='Profile Selection'):
+    def __init__(self, parent=None, stackview=None,
+                 title='Profile Selection'):
         """QToolBar providing profile tools for an image or a stack of images.
 
         :param parent: the parent QWidget
-        :param plot: :class:`PlotWindow` instance on which to operate.
+        :param stackview: :class:`StackView` instance on which to operate.
         :param str title: See :class:`QToolBar`.
         :param parent: See :class:`QToolBar`.
         """
-        from .PlotWindow import Plot1D, Plot2D      # noqa
-        super(Profile3DToolBar, self).__init__(parent=parent, plot=plot,
+        # TODO: add param profileWindow (specify the plot used for profiles)
+        super(Profile3DToolBar, self).__init__(parent=parent,
+                                               plot=stackview.getPlot(),
                                                title=title)
-        # create the main widget
-        self.ndProfileWindow = qt.QWidget()
-        self.ndProfileWindow.setWindowTitle('Profile window')
-        self.ndProfileWindow.setLayout(qt.QVBoxLayout())
-        self._profileWindow1D = Plot1D(parent=self.ndProfileWindow)
-        self._profileWindow2D = Plot2D(parent=self.ndProfileWindow)
-        self.ndProfileWindow.layout().addWidget(self._profileWindow1D)
-        self.ndProfileWindow.layout().addWidget(self._profileWindow2D)
-        # create the 3D toolbar
-        self.__create3DProfileAction()
-
-        # connect to remove the profile line (manage close event)
-        self.ndProfileWindow.installEventFilter(self)
-
-        # filter hide event when received (manage show and hide event)
-        self._profileWindow1D.installEventFilter(self)
-        self._profileWindow2D.installEventFilter(self)
-
-    def __create3DProfileAction(self):
-        """Initialize the Profile3DAction action
-        """
+        self.stackView = stackview
+        """:class:`StackView` instance"""
 
         self.profile3dAction = ProfileToolButton(
             parent=self, plot=self.plot)
-
-        # initial profile dimension is 3D
         self.profile3dAction.computeProfileIn2D()
-        self._profileDimension = 2
-        self._profileWindow1D.hide()
-        self._profileWindow2D.hide()
-
         self.profile3dAction.setVisible(True)
         self.addWidget(self.profile3dAction)
+        self.profile3dAction.sigDimensionChanged.connect(self._setProfileType)
 
-        self.profile3dAction.sigDimensionChanged.connect(self._setProfileDimension)
-        self._setProfileDimension(self._profileDimension)
+        # create the 3D toolbar
+        self._profileType = None
+        self._setProfileType(2)
 
-    def _browseActionTriggered(self, checked):
-        """Handle browse action mode triggered by user.
-        This is overloaded from :class:`ProfileToolBar` to hide
-        :attr:`ndProfileWindow` instead of :attr:`profileWindow`."""
-        if checked:
-            self.clearProfile()
-            self.plot.setInteractiveMode('zoom', source=self)
-            self.ndProfileWindow.hide()
+    def _setProfileType(self, dimensions):
+        """Set the profile type: "1D" for a curve (profile on a single image)
+        or "2D" for an image (profile on a stack of images).
 
-    def eventFilter(self, qobject, event):
-        """Observe the show and hide events of the widgets related to
-        the profile plot (a container widget, a Plot1D and a Plot2D)
-
-        :param qobject: the observed object
-        :param event: the event received by qobject
+        :param int dimensions: 1 for a "1D" profile or 2 for a "2D" profile
         """
-        if event.type() in (qt.QEvent.Close, qt.QEvent.Hide):
-            # when the container widget is closed/hidden, clear the profile
-            if qobject is self.ndProfileWindow:
-                self.clearProfile()
-
-            # else if both the plot windows are closed/hidden,
-            # make sure the container widget is hidden as well
-            elif (qobject is self._profileWindow1D and self._profileWindow2D.isHidden() or
-                  qobject is self._profileWindow2D and self._profileWindow1D.isHidden()):
-                    self.ndProfileWindow.hide()
-
-
-        return qt.QToolBar.eventFilter(self, qobject, event)
-
-    def setChildVisibility(self):
-        if self._profileDimension is 1:
-            self._profileWindow1D.setVisible(True)
-            self._profileWindow2D.setVisible(False)
-        elif self._profileDimension is 2:
-            self._profileWindow1D.setVisible(False)
-            self._profileWindow2D.setVisible(True)
-
-    def _setProfileDimension(self, dimension):
-        """Set the dimension in which we want to compute the profile.
-        Valid values are 1 and 2 for now
-
-        :param int dimension: dimension of the profile
-        """
-        self._setActiveProfileWindow(dimension)
-        profileIsVisible = self.ndProfileWindow.isVisible()
-        if dimension is 2:
-            if profileIsVisible:
-                self._profileWindow1D.hide()
-                self._profileWindow2D.show()
-        elif dimension is 1:
-            if profileIsVisible:
-                self._profileWindow2D.hide()
-                self._profileWindow1D.show()
+        # fixme this assumes that we created _profileMainWindow
+        self._profileType = "1D" if dimensions == 1 else "2D"
+        self.getProfileMainWindow().setProfileType(self._profileType)
         self.updateProfile()
 
-    def _setActiveProfileWindow(self, dimension):
-        """Set the active profile window depending on the dimension of
-        the profile
-
-        :param int dimension: dimension of the profile"""
-        self._profileDimension = dimension
-        if dimension is 2:
-            self.profileWindow = self._profileWindow2D
-        elif dimension is 1:
-            self.profileWindow = self._profileWindow1D
-        else:
-            self.profileWindow = None
-
     def updateProfile(self):
         """Method overloaded from :class:`ProfileToolBar`,
         to pass the stack of images instead of just the active image.
 
         In 1D profile mode, use the regular parent method.
         """
-        if self._profileDimension is 1:
+        if self._profileType == "1D":
             super(Profile3DToolBar, self).updateProfile()
-        elif self._profileDimension is 2:
-            stackData = self.plot.getStack(copy=False,
-                                           returnNumpyArray=True)
+        elif self._profileType == "2D":
+            stackData = self.stackView.getCurrentView(copy=False,
+                                                      returnNumpyArray=True)
             if stackData is None:
                 return
             self.plot.remove(self._POLYGON_LEGEND, kind='item')
-            self.profileWindow.clear()
-            self.profileWindow.setGraphTitle('')
-            self.profileWindow.setGraphXLabel('X')
-            self.profileWindow.setGraphYLabel('Y')
+            self.getProfilePlot().clear()
+            self.getProfilePlot().setGraphTitle('')
+            self.getProfilePlot().getXAxis().setLabel('X')
+            self.getProfilePlot().getYAxis().setLabel('Y')
 
             self._createProfile(currentData=stackData[0],
-                                params=stackData[1])
+                                origin=stackData[1]['origin'],
+                                scale=stackData[1]['scale'],
+                                colormap=stackData[1]['colormap'],
+                                z=stackData[1]['z'])
         else:
-            raise ValueError("Can't compute profile for data in %s" %
-                             str(self._profileDimension))
-
-    def _showProfileWindow(self):
-        """If profile window was created in this widget,
-        it tries to avoid overlapping this widget when shown
-        In Profile3DToolBar we have a widget grouping profile windows for 1D and 2D.
-        So we also have to manage this one
-        """
-        self.setChildVisibility()
-        self.ndProfileWindow.show()
-        super(Profile3DToolBar, self)._showProfileWindow()
-
-    def hideProfileWindow(self):
-        """Hide container window for profile windows.
-        """
-        self.ndProfileWindow.hide()
-
-    def getProfileWindow1D(self):
-        """Plot window used to display 1D profile curve.
-
-        :return: :class:`Plot1D`
-        """
-        return self._profileWindow1D
-
-    def getProfileWindow2D(self):
-        """Plot window used to display 2D profile image.
-
-        :return: :class:`Plot2D`
-        """
-        return self._profileWindow2D
+            raise ValueError(
+                    "Profile type must be 1D or 2D, not %s" % self._profileType)
